'use strict'
const _ = require('lodash')
const async = require('async')
const config = require('./rc')().get()

const api = require('./api.js')(config.medx.username, config.medx.password)
const store = require('./store.js')

module.exports = (key, answer, deleteAllProfiles, handleResponse) => {
  const handleData = (err, data, dropLast) => {
    if (err) return handleResponse(err)
    if (!dropLast) return addItem(key, _.assign(data, {prevAnswer: answer}))
    store.drop(key, (err) => {
      if (err) return handleResponse(err)
      addItem(key, data)
    })
  }

  const addItem = (key, data) => {
    store.setState(key, data, (err) => {
      if (err) return handleResponse(err)
      handleResponse(null, data)
    })
  }

  if (!api.isLoggedIn()) return api.login(deleteAllProfiles, handleData)

  async.series({
    state: (cb) => store.getState(key, cb),
    prevState: (cb) => store.getPrevState(key, cb)
  }, (err, res) => {
    if (err) throw err
    api.ask(answer, res.state, res.prevState, res.state.prevAnswer, key === 'cli', handleData)
  })
}
